#ifndef __BIAWEBSIDEBAR__
#define __BIAWEBSIDEBAR__
#include <string>
#include <list>
#include <iostream>
#include <fstream>
#include "biawebutil.hpp"

// classes to describe the a list of items and sidebar item containers which form part of 
// main document
namespace biaweb {
    // class to represent a sidebar item which can contain a text and link or only
    // text
    class SideBarItem {
      protected:
        // sidebar text and url
        std::string sidebar_text;
        std::string sidebar_url;
      public:
        std::string get_sidebar_text () {
            return this->sidebar_text;
        }
        void set_sidebar_text (std::string text) {
            this->sidebar_text = text;
        }
        std::string get_sidebar_url () {
            return this->sidebar_url;
        }
        void set_sidebar_url (std::string url) {
            this->sidebar_url = url;
        }
        std::string to_html ();
    
        SideBarItem (std::string text = "", std::string url = "") {
            this->sidebar_text = text;
            this->sidebar_url = url;
        }
    };

    std::string SideBarItem::to_html () {
        std::string html;
        // if url is not empty it is a link item so load the sidebar link template
        if (! this->sidebar_url.empty ()) {
            if (!this->sidebar_text.empty ())
            {
                std::ifstream tpl_linkitem ("templates/sidebarlinkitem.tpl.html");
                std::string tpl_linkitem_str ( (std::istreambuf_iterator<char> (tpl_linkitem)),
                                                (std::istreambuf_iterator<char> ()));
                tpl_linkitem.close ();
                std::unique_ptr<char[]> linktxt (new char[tpl_linkitem_str.size() 
                                                        + this->sidebar_text.size ()
                                                        + this->sidebar_url.size ()] );
                std::sprintf (linktxt.get(), tpl_linkitem_str.c_str (),
                                             this->sidebar_url.c_str (),
                                            this->sidebar_text.c_str ());
                html.append (linktxt.get ());
            }
            // no text or url - item is empty - so it should be blank
            else
                html = "";
        }
        // Non link item. Load the normal sidebar item template.
        else
        {
            std::ifstream tpl_item ("templates/sidebaritem.tpl.html");
            std::string tpl_item_str ( (std::istreambuf_iterator<char> (tpl_item)),
                                        (std::istreambuf_iterator<char> ()));
            tpl_item.close ();
            std::unique_ptr<char[]> txt (new char [tpl_item_str.size () +
                                                    this->sidebar_text.size ()]);
            std::sprintf (txt.get (), tpl_item_str.c_str(), this->sidebar_text.c_str());
            html.append (txt.get ());
        } 
        return html;
    }
    
    // Class to represent a sidebar, which contains one heading and a list of items 
    // either links or non-link items.
    class SideBar {
      protected:
        std::string sidebar_title;
        std::list <SideBarItem> items;
        
      public:
        void set_title (std::string title) {
            this->sidebar_title = title;
        }

        void add_sidebar_item (SideBarItem item) {
            items.insert (items.cend(), item);
        }

        // render the sidebar
        std::string to_html () ;
    };

    // render the sidebar to HTML representation from the template.
    std::string SideBar::to_html () {
        std::ifstream sidetpl ("templates/sidebar.tpl.html");
        std::string sidetpl_str ( ( std::istreambuf_iterator<char> (sidetpl)) , 
                                       (std::istreambuf_iterator<char> ()));
        sidetpl.close ();
        std::string listitem;
        // first get the sidebar items and render them to HTML
        for (SideBarItem item : items) {
            listitem += item.to_html ();
        }

        std::unique_ptr<char[]> tpl_final (new char[sidetpl_str.size() + 
                                                this->sidebar_title.size () +
                                                listitem.size () ]);
        
        std::string html;
        // if there are items, sidebar should be rendered
        if (items.size () > 0)
        {        
            std::sprintf (tpl_final.get (), sidetpl_str.c_str(), 
                                        this->sidebar_title.c_str(), listitem.c_str()) ;
            html.append ( tpl_final.get ());
        }
        // no items in the sidebar, render as empty string (even if it has a heading)
        // since heading becomes meaningless without items
        else 
            html = "";
        return html; 
    }
}

#endif
